## Script for analysing the output of phylogenetic factor analysis in BEAST
# Sarah-Sophie Weil
# 10/06/2020

# what you need: a .txt file with factors for species from pfa_pipeline (in Data/Traits)
# what you get: .data files with binary traits for analysis in BioGeoBEARS (in Data/Traits)

# clear environment
rm(list = ls())


#### Preparation ----
# set up working directory and output directory
wd <- "../Data/"
setwd(wd)
list.files()


library(BioGeoBEARS) # 1.1.2


sv.output <- 1
clade <- "Chamaeleonidae"






#### Create .data file for analysis in BioGeoBEARS ----

# import data from Data/Traits
factors <- read.table(paste0("Traits/Chamaeleonidae/",
                             "20201221_Chamaeleonidae_181_constrained_MSE_factors.txt"),
                      header = TRUE, stringsAsFactors = FALSE)

#### Using the first factor to do a median split (body size axis) ----
med_split <- median(factors$f1)
traits <- factors[,c(1,3)]
colnames(traits)[1] <- "Species"
traits$Fast <- 0; traits$Slow <- 0
traits$Slow[traits$f1 < med_split] <- 1
traits$Fast[traits$f1 >= med_split] <- 1
# to test if it worked
sum(traits[,c(3,4)]) # should be 181
plot(traits$f1, traits$Slow, col = "red")
plot(traits$f1, traits$Fast, col = "blue")

# put species as rownames and remove species names and factor columns
rownames(traits) <- traits$Species
traits <- traits[sort(traits$Species),]
traits <- traits[,-c(1:2)]
if(sv.output == 1){
  tmp <- define_tipranges_object(traits)
  save_tipranges_to_LagrangePHYLIP(tmp, paste("Traits/", clade, "/", clade,
                                              "_traits_body_sizePFAtest.data", sep = ""),
                                   areanames = colnames(tmp@df))
  rm(tmp)
}



#### Median split on 2nd factor (life-history axis) ----
med_split <- median(factors$f2)
traits <- factors[,c(1,4)]
colnames(traits)[1] <- "Species"
traits$Fast <- 0; traits$Slow <- 0
traits$Slow[traits$f2 > med_split] <- 1 # high values are long SM, long GT -> slow LH
traits$Fast[traits$f2 <= med_split] <- 1
# to test if it worked
sum(traits[,c(3,4)]) # should be 181
plot(traits$f2, traits$Slow, col = "red")
plot(traits$f2, traits$Fast, col = "blue")

# put species as rownames and remove species names and factor columns
rownames(traits) <- traits$Species
traits <- traits[sort(traits$Species),]
traits <- traits[,-c(1:2)]
if(sv.output == 1){
  tmp <- define_tipranges_object(traits)
  save_tipranges_to_LagrangePHYLIP(tmp, paste("Traits/", clade, "/", clade,
                                              "_traits_LH.data", sep = ""),
                                   areanames = colnames(tmp@df))
  rm(tmp)
}





#### First 25% are considered dispersal-prone ("Fast") ----
med_split <- sort(factors$f2)[ceiling(0.25*dim(factors)[1])]
traits <- factors[,c(1,4)]
colnames(traits)[1] <- "Species"
traits$Fast <- 0; traits$Slow <- 0
traits$Slow[traits$f2 > med_split] <- 1
traits$Fast[traits$f2 <= med_split] <- 1
# to test if it worked
sum(traits[,c(3,4)]) # should be 181
plot(traits$f2, traits$Slow, col = "red")
plot(traits$f2, traits$Fast, col = "blue")
if(sv.output == 1){
  save(traits, file = paste("Traits/", clade, "/", clade, "_traits_LH_first_25.RData",
                            sep = ""))
}

# put species as rownames and remove species names and factor columns
rownames(traits) <- traits$Species
traits <- traits[sort(traits$Species),]
traits <- traits[,-c(1:2)]
if(sv.output == 1){
  tmp <- define_tipranges_object(traits)
  save_tipranges_to_LagrangePHYLIP(tmp, paste("Traits/", clade, "/", clade, 
                                              "_traits_LH_first_25.data", sep = ""),
                                   areanames = colnames(tmp@df))
  rm(tmp)
}





#### First 75% are dispersal-prone ----
med_split <- sort(factors$f2)[ceiling(0.75*dim(factors)[1])]
traits <- factors[,c(1,4)]
colnames(traits)[1] <- "Species"
traits$Fast <- 0; traits$Slow <- 0
traits$Slow[traits$f2 > med_split] <- 1
traits$Fast[traits$f2 <= med_split] <- 1
# to test if it worked
sum(traits[,c(3,4)]) # should be 181
plot(traits$f2, traits$Slow, col = "red")
plot(traits$f2, traits$Fast, col = "blue")
if(sv.output == 1){
  save(traits, file = paste("Traits/", clade, "/", clade, "_traits_LH_first_75.RData", sep = ""))
}

# put species as rownames and remove species names and factor columns
rownames(traits) <- traits$Species
traits <- traits[sort(traits$Species),]
traits <- traits[,-c(1:2)]
if(sv.output == 1){
  tmp <- define_tipranges_object(traits)
  save_tipranges_to_LagrangePHYLIP(tmp, paste("Traits/", clade, "/", clade, 
                                              "_traits_LH_first_75.data", sep = ""),
                                   areanames = colnames(tmp@df))
  rm(tmp)
}




#### Extreme LH ----
## assigning the fastest *and* slowest 25% of species as "fast" (dispersal-prone) species
med_split_low <- sort(factors$f2)[ceiling(0.25*dim(factors)[1])]
med_split_high <- sort(factors$f2, decreasing = TRUE)[ceiling(0.25*dim(factors)[1])]
traits <- factors[,c(1,4)]
colnames(traits)[1] <- "Species"
traits$Fast <- 0; traits$Slow <- 0
traits$Slow[traits$f2 > med_split_low & traits$f2 < med_split_high] <- 1
traits$Fast[traits$f2 <= med_split_low | traits$f2 >= med_split_high] <- 1
# to test if it worked
sum(traits[,c(3,4)]) # should be 181
plot(traits$f2, traits$Slow, col = "red")
plot(traits$f2, traits$Fast, col = "blue")
if(sv.output == 1){
  save(traits, file = paste("Traits/", clade, "/", clade, 
                            "_traits_LH_first_and_last_25.RData", sep = ""))
}

# put species as rownames and remove species names and factor columns
rownames(traits) <- traits$Species
traits <- traits[sort(traits$Species),]
traits <- traits[,-c(1:2)]
if(sv.output == 1){
  tmp <- define_tipranges_object(traits)
  save_tipranges_to_LagrangePHYLIP(tmp, paste("Traits/", clade, "/", clade,
                                              "_traits_LH_first_and_last_25.data", sep = ""),
                                   areanames = colnames(tmp@df))
  rm(tmp)
}




#### Create a combined trait ----
# where species are only considered dispersal-prone if they reunite all dispersal traits 
# load habitat data
tmplines <- scan(file = "Traits/Chamaeleonidae/Chamaeleonidae_traits_habitat.data",
                 what = "character", sep="\n")
tmplines <- tmplines[2:length(tmplines)]
habitat <- as.data.frame(tmplines)
rm(tmplines)
colnames(habitat) <- "Species"
habitat$coast <- habitat$Species
habitat$Species <- sub('\\\t.*', '', habitat$Species)
habitat$coast <- sub('.*\t', '', habitat$coast)
habitat$interior <- habitat$coast
habitat$coast <- substr(habitat$coast, 1, 1)     
habitat$interior <- substr(habitat$interior, 2, 2)     
habitat$coast <- as.numeric(habitat$coast)
habitat$interior <- as.numeric(habitat$interior)
sum(habitat[,c(2,3)]) # should be 181


all_traits <- merge(habitat,factors[,c("taxon", "f1", "f2")], 
                    by.x = "Species", by.y = "taxon")
## add other traits
all_traits$body <- 0
med_split <- median(all_traits$f1)
all_traits$body[all_traits$f1 >= med_split] <- 1
all_traits$LH <- 0
med_split_low <- sort(all_traits$f2)[ceiling(0.25*dim(all_traits)[1])]
med_split_high <- sort(all_traits$f2, decreasing = TRUE)[ceiling(0.25*dim(all_traits)[1])]
all_traits$LH[all_traits$f2 <= med_split_low | all_traits$f2 >= med_split_high] <- 1
rm(med_split,med_split_high,med_split_low,habitat)

all_traits$Fast_newcombo <- 0
all_traits$Slow_newcombo <- 0
all_traits$Fast_newcombo[rowSums(all_traits[,c("coast", "body", "LH")]) == 3] <- 1
all_traits$Slow_newcombo[all_traits$Fast_newcombo == 0] <- 1


combo33 <- all_traits[,c("Species","Fast_newcombo", "Slow_newcombo")]
# put species as rownames and remove species names
rownames(combo33) <- combo33$Species
combo33 <- combo33[sort(combo33$Species),]
combo33 <- combo33[,-1]
if(sv.output == 1){
  tmp <- define_tipranges_object(combo33)
  save_tipranges_to_LagrangePHYLIP(tmp, paste("Traits/", clade, "/", clade,
                                              "_traits_combo33_new.data", sep = ""),
                                   areanames = colnames(tmp@df))
  rm(tmp)
}