## Analysing the output from the cluster
# Sarah-Sophie Weil, 20/02/2021

# Here the base models DEC, DIVA and BAYAREA and the +x version for the four different MDMMs
# are compared


## what you need:
# - Rdata objects that are outputs of a BioGeoBEARS run 
# (which are in the folder Data/BioGeoBEARS)




# clear environment
rm(list = ls())


#### Preparation ----
# set up working directory and output directory
wd <- "../Data"
output_data <- "../Data"
output_results <- "../Outputs"
setwd(wd)
list.files()

library(BioGeoBEARS)  # 1.1.2
library(readODS)      # 1.7.0
library(phylolm)      # 2.6
library(sf)           # 0.9.8
library(viridis)      # 0.5.1
library(ggplot2)      # 3.3.3



sv.output <- 1 # save if 1
clade <- "Chamaeleonidae" # this is going to be added before each output






#### Loading required files (prepared in Script1 and 2) ----
# 
# path to tree
trfn <- paste(output_data, "/BioGeoBEARS/", clade, "/", clade, 
              "_tree_final_binary.nwk", sep = "")
# load it
tree <- read.tree(file = trfn)

plot(tree)
axisPhylo(side = 1)
length(tree$tip.label)

# # load geography file
# geogfn <- paste(output_data, "/Geographies/", clade, "/", clade, "_geography.data", sep = "")
# moref(geogfn)   # Look at the raw geography text file:
# 
# # Look at your geographic range data:
# tipranges <- getranges_from_LagrangePHYLIP(lgdata_fn = geogfn)





#### Set up parameters and filenames ----
# doing this in a for-loop to create same filenames for DEC, DIVALIKE and BAYAREALIKE
short <- c("DEC", "DIV", "BAY") 
long <- c("DEC", "DIVALIKE", "BAYAREALIKE")
# these are the subfolders where the results are stored
subfolder <- c("MDMM_null",
               "MDMM_full05", 
               "MDMM_medium",
               "MDMM_simple",
               "MDMM_full01")



restable <- NULL

# storing the filenames in lists cause otherwise it's cluttering the workspace
resfn_all <- vector(mode = "list", length = length(subfolder))
for(i in 1:length(resfn_all)){
  resfn_all[[i]] <- vector(mode = "list", length = 3)
}
names(resfn_all) <- c("MDMM_null", "MDMM_full05", "MDMM_medium", "MDMM_simple",
                      "MDMM_full01")
for(j in 1:length(resfn_all)){
  for(i in 1:length(short)){
    resfn_all[[j]][[i]][1] <- paste0(output_data, "/BioGeoBEARS/", clade, "/", 
                                     subfolder[j], "/", clade, "_", long[i], ".Rdata")
    resfn_all[[j]][[i]][2] <- paste0(output_data, "/BioGeoBEARS/", clade, "/", 
                                     subfolder[j], "/", clade, "_", long[i], "+x.Rdata")
    
    names(resfn_all[[j]][[i]]) <- c(paste0("resfn_", short[i]), paste0("resfn_", 
                                                                       short[i], "x"))
    #names(resfn_all[[j]][[i]]) <- c(paste0("resfn_", short[i]))
    
  }
  names(resfn_all[[j]]) <- long
}





#### Calculating summary statistics ----
results_object <- vector(mode = "list", length = length(subfolder))
for(i in 1:length(results_object)){
  results_object[[i]] <- vector(mode = "list", length = 3)
}
names(results_object) <- names(resfn_all)
for(i in 1:length(resfn_all)){
  for(j in 1:length(resfn_all[[i]])){
    for(k in 1:length(resfn_all[[i]][[j]])){
      load(resfn_all[[i]][[j]][k])
      results_object[[i]][[j]][[k]] <- res
      
      tmp <- extract_params_from_BioGeoBEARS_results_object(results_object = 
                                                              results_object[[i]][[j]][[k]], 
                                                            returnwhat = "table",
                                                            addl_params = c("x"), 
                                                            paramsstr_digits = 4)
      
      rownames(tmp) <- paste0(names(results_object[i]), "_", 
                              sub("resfn_", "", names(resfn_all[[i]][[j]])[k]))
      
      restable <- rbind(restable, tmp)
    }
    names(results_object[[i]][[j]]) <- names(resfn_all[[i]][[j]])
  }
  names(results_object[[i]]) <- names(resfn_all[[i]])
}
rm(i, j, k, res, tmp)







# With AICcs -- factors in sample size
samplesize <- length(tree$tip.label)
AICtable <- calc_AICc_column(LnL_vals = restable$LnL, 
                             nparam_vals = restable$numparams, samplesize = samplesize)
restable2 <- cbind(restable, AICtable)
restable_AICc_rellike <- AkaikeWeights_on_summary_table(restable = restable2, 
                                                        colname_to_use = "AICc")
rm(AICtable, samplesize, restable2, restable)

# add another column with rounded %values to be able to actually read something
restable_AICc_rellike$AICc_wt100 <- round(restable_AICc_rellike$AICc_wt*100, 2)


if(sv.output == 1){
  
  # write in ods
  write_ods(restable_AICc_rellike, paste(output_results, "/", clade, 
                                         "/Tables/", clade, "_AICc_table_MDMMs.ods", 
                                         sep = ""),
            sheet = "all models", row_names = TRUE, append = FALSE)
  save(restable_AICc_rellike, file = paste(output_results, "/", 
                                           clade, "/Tables/", clade, 
                                           "_AICc_table_MDMMs.Rdata", sep = ""))
  
}







#### Plot results ----
load(paste(output_results, "/", clade, "/Tables/", clade,
           "_AICc_table_MDMMs.Rdata", sep = ""))
# first, add columns with levels: MDMM, base model, param
restable_AICc_rellike$MDMM <- sub('^([^_]+_[^_]+).*', '\\1', rownames(restable_AICc_rellike))
restable_AICc_rellike$base_model <- substr(sub('.*_', '', 
                                               rownames(restable_AICc_rellike)), 
                                           start = 1, stop = 3)
restable_AICc_rellike$params <- NA
restable_AICc_rellike$params[restable_AICc_rellike$numparams == 2] <- "Null"
restable_AICc_rellike$params[restable_AICc_rellike$numparams == 3] <- "+x"
# transform columns into factors and define order
restable_AICc_rellike$MDMM <- factor(restable_AICc_rellike$MDMM, 
                                     levels = c("MDMM_null", "MDMM_simple", "MDMM_medium",
                                                "MDMM_full05", "MDMM_full01"))
restable_AICc_rellike$base_model <- factor(restable_AICc_rellike$base_model, 
                                           levels = c("DEC", "DIV", "BAY"))
restable_AICc_rellike$params <- factor(restable_AICc_rellike$params, levels = 
                                         c("Null", "+x"))



# do some nice barplots
# first, create colour palette
c.palette <- viridis(6)[seq(6, 1, -2)]    # defining 3 colours for DEC, BAY and DIVA
names(c.palette) <- levels(restable_AICc_rellike$base_model)   


(hist <- ggplot(restable_AICc_rellike, aes(x = params, y = LnL, fill = base_model)) + 
    geom_bar(stat = "identity", position  = position_stack(reverse = TRUE), width = 0.9) + 
    scale_fill_manual(values = c.palette,                        # using our palette here
                      name = "Base model") +
    geom_text(aes(label = round(LnL, 2)), position = 
                position_stack(vjust = 0.5, reverse = TRUE), size = 3) +
    facet_wrap(~MDMM, nrow = 1, strip.position = "bottom") + 
    theme_bw() +
    xlab("Parameters") + 
    theme(
      panel.spacing = unit(0, "lines"),
      axis.text = element_text(size = 12), 
      axis.title = element_text(size = 12), 
      plot.title = element_text(size = 14, hjust = 0.5, face = "bold")))

# changing orders of stacks individually based on values
# first: add a column where order is specified
restable_AICc_rellike$ordering <- NA
for(i in 1:length(unique(restable_AICc_rellike$MDMM))){
  for(j in 1:length(unique(restable_AICc_rellike$params))){
    tmp <- restable_AICc_rellike[restable_AICc_rellike$MDMM == 
                                   unique(restable_AICc_rellike$MDMM)[i] & 
                                  restable_AICc_rellike$params == 
                                   unique(restable_AICc_rellike$params)[j],]
    tmp <- tmp[order(tmp$LnL, decreasing = TRUE),]
    for(k in 1:nrow(tmp)){
      restable_AICc_rellike$ordering[rownames(restable_AICc_rellike) == rownames(tmp)[k]] <- k
    }
   
  }
}
(p <- ggplot(restable_AICc_rellike, aes(x = params, y = LnL, 
                                        fill = reorder(base_model,-ordering))) + 
    geom_bar(stat = "identity", 
             position = position_stack(vjust = 0.5, reverse = FALSE), width = 0.9,
             data = restable_AICc_rellike[restable_AICc_rellike$params == "+x" & 
                                            restable_AICc_rellike$MDMM != "MDMM_full01",]) +
    geom_text(aes(label = round(LnL, 2)), 
              position = position_stack(vjust = 0.5, reverse = FALSE), size = 3,
              data = restable_AICc_rellike[restable_AICc_rellike$params == "+x" & 
                                             restable_AICc_rellike$MDMM != "MDMM_full01",]) +
    geom_bar(stat = "identity", 
             position = position_stack(vjust = 0.5, reverse = FALSE), width = 0.9,
             data = restable_AICc_rellike[restable_AICc_rellike$params == "Null" & 
                                            restable_AICc_rellike$MDMM != "MDMM_full01",]) +
    geom_text(aes(label = round(LnL, 2)), 
              position = position_stack(vjust = 0.5, reverse = FALSE), size = 3,
              data = restable_AICc_rellike[restable_AICc_rellike$params == "Null" & 
                                             restable_AICc_rellike$MDMM != "MDMM_full01",]) +
    
    # need to do MDMM01 separately because things are inverted there
    geom_bar(stat = "identity", position = position_stack(vjust = 0.5, reverse = FALSE), 
             width = 0.9,
             data = restable_AICc_rellike[restable_AICc_rellike$params == "Null" & 
                                            restable_AICc_rellike$MDMM == "MDMM_full01",]) +
    geom_text(aes(label = round(LnL, 2)), 
              position = position_stack(vjust = 0.5, reverse = FALSE), size = 3,
              data = restable_AICc_rellike[restable_AICc_rellike$params == "Null" & 
                                             restable_AICc_rellike$MDMM == "MDMM_full01",]) +
    geom_bar(stat = "identity", position = 
               position_stack(vjust = 0.5, reverse = FALSE), width = 0.9,
             data = restable_AICc_rellike[restable_AICc_rellike$params == "+x" & 
                                            restable_AICc_rellike$MDMM == "MDMM_full01",]) +
    geom_text(aes(label = round(LnL, 2)), 
              position = position_stack(vjust = 0.5, reverse = FALSE), size = 3,
              data = restable_AICc_rellike[restable_AICc_rellike$params == "+x" &
                                             restable_AICc_rellike$MDMM == "MDMM_full01",]) +
    
    scale_fill_manual(values = c.palette,                        
                      name = "Base model") +
    facet_wrap(~MDMM, nrow = 1, strip.position = "bottom") +
    theme_bw() +
    xlab("Parameters") + 
    theme(
      panel.spacing = unit(0, "lines"),
      axis.text = element_text(size = 12), 
      axis.title = element_text(size = 12), 
      plot.title = element_text(size = 14, hjust = 0.5, face = "bold")))



if(sv.output == 1){
  ggsave(filename = paste(output_results, "/", clade, "/Tables/", 
                          clade, "_AICc_table_MDMMs.png", sep = ""), 
         plot = p, 
         width = 7, height = 5, dpi = 300)
}


