## Analysing the output from the cluster
# Sarah-Sophie Weil, 24/11/2020



## what you need:
# - Rdata objects that are outputs of a BioGeoBEARS run 
# (which are in the folder Data/BioGeoBEARS)



## what you get (all in Outputs):
# - tables of model comparisons with AICc. 
# - tables of output parameters of models
# - estimated ancestral regions and trait states of all models in a pdf



# clear environment
rm(list = ls())


#### Preparation ----
# set up working directory and output directory
wd <- "../Data"
output_data <- "../Data"
output_results <- "../Outputs"
setwd(wd)
list.files()

library(BioGeoBEARS) # 1.1.2
library(readODS)     # 1.7.0
library(phylolm)     # 2.6
library(sf)          # 0.9.8, GEOS 3.8.0, GDAL 3.0.4, PROJ 6.3.1
library(viridis)     # 0.5.1
library(cladoRcpp)   # 0.15.1, for function rcpp_areas_list_to_states_list

# source the function I adapted from plot_BioGeoBEARS_results
source("../Scripts/5_4_1_my_plot_BioGeoBEARS_result_function.R")



sv.output <- 1 # save if 1
clade <- "Chamaeleonidae" # this is going to be added before each output
model <- 1 # 1 for DEC




#### Loading required files (prepared in Script1 and 2) ----

# path to tree
trfn <- paste(output_data, "/BioGeoBEARS/", clade, "/", clade, 
              "_tree_final_binary.nwk", sep = "")
# load it
tree <- read.tree(file = trfn)

plot(tree)
axisPhylo(side = 1)
length(tree$tip.label)

# load geography file
geogfn <- paste(output_data, "/Geographies/", clade, "/", clade, "_geography.data", sep = "")
moref(geogfn)   # Look at the raw geography text file:

# Look at your geographic range data:
tipranges <- getranges_from_LagrangePHYLIP(lgdata_fn = geogfn)




#### calculate everything for a set of different traits ----
# (with a leading _)
traits_incl <- c("_habitat", "_body_sizePFA", "_LH", "_LH_first_25", "_LH_first_75",
                 "_LH_first_and_last_25", "_combo33_new")
subfolder <- "MDMM_full01/" # if the results are stored in a subfolder

for(k in traits_incl){
  #### Set up parameters and filenames ----
  # doing this in a for-loop to create same filenames for DEC, DIVALIKE and BAYAREALIKE
  short <- c("DEC", "DIV", "BAY") 
  long <- c("DEC", "DIVALIKE", "BAYAREALIKE")
  # storing the filenames in lists cause otherwise it's cluttering the workspace
  resfn_all <- analysis_titletxt <- vector(mode = "list", length = 3)
  names(resfn_all) <- names(analysis_titletxt) <- long
  
  for(i in 1:length(short)){
    resfn_all[[i]][1] <- paste0(output_data, "/BioGeoBEARS/", clade, "/", 
                                clade, "_", long[i], 
                                "+traits_only", k, ".Rdata")
    resfn_all[[i]][2] <- paste0(output_data, "/BioGeoBEARS/", clade, "/",
                                clade, "_", long[i], ".Rdata")
    resfn_all[[i]][3] <- paste0(output_data, "/BioGeoBEARS/", clade, "/",
                                clade, "_", long[i], "+x.Rdata")
    resfn_all[[i]][4] <- paste0(output_data, "/BioGeoBEARS/", clade, "/", 
                                clade, "_", long[i], 
                                "+m2", k, ".Rdata")
    resfn_all[[i]][5] <- paste0(output_data, "/BioGeoBEARS/", clade, "/", 
                                clade, "_", long[i],
                                "+m2+x", k, ".Rdata")
    names(resfn_all[[i]]) <- c(paste0("resfn_", short[i], "traits", k), 
                               paste0("resfn_", short[i]), 
                               paste0("resfn_", short[i], "x"), 
                               paste0("resfn_", short[i], "m2", k),
                               paste0("resfn_", short[i], "m2x", k))
    
    analysis_titletxt[[i]][1] <- paste0("BioGeoBEARS ", long[i], " on ", 
                                        clade, " stratified")
    analysis_titletxt[[i]][2] <- paste0("BioGeoBEARS ", long[i], "+x on ", 
                                        clade, " stratified")
    analysis_titletxt[[i]][3] <- paste0("BioGeoBEARS ", long[i], "+m2", k, 
                                        " on ", clade, " stratified")
    analysis_titletxt[[i]][4] <- paste0("BioGeoBEARS ", long[i], "+m2+x", k, 
                                        " on ", clade, " stratified")
    names(analysis_titletxt[[i]]) <- c(paste0("analysis_titletxt_", short[i]), 
                                       paste0("analysis_titletxt_", short[i], "x"),  
                                       paste0("analysis_titletxt_", short[i], "m2", k), 
                                       paste0("analysis_titletxt_", short[i], "m2x", k))
    
  }
  
  
  
  rm(long, i)
  
  pdffn_DEC <- paste(output_results, "/", clade, "/", clade, "_DEC.pdf", sep = "")
  pdffn_DIVALIKE <- paste(output_results, "/", clade, "/", clade, "_DIVALIKE.pdf", sep = "")
  pdffn_BAYAREALIKE <- paste(output_results, "/", clade, "/", clade, 
                             "_BAYAREALIKE.pdf", sep = "")
  pdffn1 <- list(pdffn_DEC, pdffn_DIVALIKE, pdffn_BAYAREALIKE)
  rm(pdffn_DEC, pdffn_DIVALIKE, pdffn_BAYAREALIKE)
 
  
  
  
  
  
  #### Calculating summary statistics ----
  for(i in model){
    for(j in 1:length(resfn_all[[i]])){
      load(sub("Chamaeleonidae/", paste0("Chamaeleonidae/", subfolder), resfn_all[[i]][j]))
      #load(resfn_all[[i]][j])
      assign(sub("resfn_", "res", names(resfn_all[[i]])[j]), res)
    }
  }
  rm(i,j, res)
  
  # adding LnL of just trait run to trait-independent models to see the difference m2 makes
  LnL_traits <- list(get_LnL_from_BioGeoBEARS_results_object(eval(
    as.symbol(paste0("res", short[model], "traits", k)))))
  LnL_0 <- list(get_LnL_from_BioGeoBEARS_results_object(eval(
    as.symbol(paste0("res", short[model])))) + LnL_traits[[model]])
  LnL_X <- list(get_LnL_from_BioGeoBEARS_results_object(eval(
    as.symbol(paste0("res", short[model], "x")))) + LnL_traits[[model]])
  LnL_M <- list(get_LnL_from_BioGeoBEARS_results_object(eval(
    as.symbol(paste0("res", short[model], "m2", k)))))
  LnL_MX <- list(get_LnL_from_BioGeoBEARS_results_object(eval(
    as.symbol(paste0("res", short[model], "m2x", k)))))
  
  results_object_traits <- list(eval(as.symbol(paste0("res", short[model], "traits", k))))
  results_object_0 <- list(eval(as.symbol(paste0("res", short[model]))))
  results_object_X <- list(eval(as.symbol(paste0("res", short[model], "x"))))
  results_object_M <- list(eval(as.symbol(paste0("res", short[model], "m2", k))))
  results_object_MX <- list(eval(as.symbol(paste0("res", short[model], "m2x", k))))
  
  numparams0 <- 4; numparamsX <- 5; numparamsM <- 5; numparamsMX <- 6
  rm(list = c(paste0("res", short[model]), paste0("res", short[model], "m2", k),
              paste0("res", short[model], "x"), paste0("res", short[model], "m2x", k), 
              paste0("res", short[model], "traits", k)))
  

  
  
  # Model weights of all six models
  # prepare results table
  restable <- NULL
  # first: all models in following order: DEC, DEC+m2, DEC+x, DEC+m2+x (same for BAY and DIVA)

  tmp <- extract_params_from_BioGeoBEARS_results_object(
    results_object = results_object_0[[1]], returnwhat = "table",
    addl_params = c("x", "t12", "t21", "m1", "m2"), 
    paramsstr_digits = 4)
  
  
  tmp1 <- extract_params_from_BioGeoBEARS_results_object(
    results_object = results_object_traits[[1]], returnwhat = "table",
    paramsstr_digits = 4)
  colnames(tmp1)[colnames(tmp1) == "LnL"] <- "trait LnL"
  # adding the trait-independent run
  tmp$numparams <- tmp$numparams + 2 # (because I'm adding t12 and t21)
  tmp$t12 <- tmp1$t12
  tmp$t21 <- tmp1$t21
  tmp$m1 <- 1
  tmp$m2 <- 1
  tmp$'joint LnL' <- tmp$LnL + tmp1$`trait LnL` # adding also LnL
  colnames(tmp)[colnames(tmp) == "LnL"] <- "geog LnL"
  tmp <- cbind(tmp[colnames(tmp) == "geog LnL"], tmp1[colnames(tmp1) == "trait LnL"],
               tmp[colnames(tmp) == "joint LnL"], tmp[2:(dim(tmp)[2]-1)])
  
  rownames(tmp) <- sub("resfn_", "", names(resfn_all[[model]])[2])
 
  restable <- rbind(restable, tmp)
  
  tmp <- extract_params_from_BioGeoBEARS_results_object(
    results_object = results_object_M[[1]], returnwhat = "table",
    addl_params = c("x", "m1"), paramsstr_digits = 4)
  
  colnames(tmp)[colnames(tmp) == "LnL"] <- "joint LnL"
  tmp$'geog LnL' <- NA
  tmp$'trait LnL' <- NA
 
  rownames(tmp) <- sub("resfn_", "", names(resfn_all[[1]])[4])

  restable <- rbind(restable, tmp)

  tmp <- extract_params_from_BioGeoBEARS_results_object(
    results_object = results_object_X[[1]], returnwhat = "table",
    addl_params = c("t12", "t21", "m1", "m2"), paramsstr_digits = 4)
  
  tmp1 <- extract_params_from_BioGeoBEARS_results_object(
    results_object = results_object_traits[[1]], returnwhat = "table",
    paramsstr_digits = 4)
  colnames(tmp1)[colnames(tmp1) == "LnL"] <- "trait LnL"
  # adding the trait-independent run
  tmp$numparams <- tmp$numparams + 2 # (because I'm adding t12 and t21)
  tmp$t12 <- tmp1$t12
  tmp$t21 <- tmp1$t21
  tmp$m1 <- 1
  tmp$m2 <- 1
  tmp$'joint LnL' <- tmp$LnL + tmp1$`trait LnL` # adding also LnL
  colnames(tmp)[colnames(tmp) == "LnL"] <- "geog LnL"
  tmp <- cbind(tmp[colnames(tmp) == "geog LnL"], tmp1[colnames(tmp1) == "trait LnL"],
               tmp[colnames(tmp) == "joint LnL"], tmp[2:(dim(tmp)[2]-1)])
  
  rownames(tmp) <- sub("resfn_", "", names(resfn_all[[1]])[3])
  
  restable <- rbind(restable, tmp)
  
  tmp <- extract_params_from_BioGeoBEARS_results_object(
    results_object = results_object_MX[[1]], returnwhat = "table",
    addl_params = c("m1"), paramsstr_digits = 4)
  
  colnames(tmp)[colnames(tmp) == "LnL"] <- "joint LnL"
  tmp$'geog LnL' <- NA
  tmp$'trait LnL' <- NA
  
  rownames(tmp) <- sub("resfn_", "", names(resfn_all[[1]])[5])
  
  restable <- rbind(restable, tmp)
  
  rm(tmp, tmp1)
  

  # With AICcs -- factors in sample size
  samplesize <- length(tree$tip.label)
  AICtable <- calc_AICc_column(LnL_vals = restable$`joint LnL`, 
                               nparam_vals = restable$numparams, samplesize = samplesize)
  restable2 <- cbind(restable, AICtable)
  restable_AICc_rellike <- AkaikeWeights_on_summary_table(restable = restable2, 
                                                          colname_to_use = "AICc")
 
  rm(AICtable, samplesize, restable2)
  
  # add another column with rounded %values to be able to actually read something
  restable_AICc_rellike$AICc_wt100 <- round(restable_AICc_rellike$AICc_wt*100, 2)
  
  
 
  
  # Also save to text files
  if(sv.output == 1){
   
    # write in ods
    write_ods(restable_AICc_rellike, paste(output_results, "/",
                                           clade, "/Tables/", clade, "_AICc_tables", k, ".ods", sep = ""),
              sheet = "all models", row_names = TRUE, append = FALSE)
    
   
  }
  
  rm(numparamsM, numparams0, numparamsX, numparamsMX)
  
  
}
  
  
  
    
#### Plot best models ----

# load bioregions to get Names of regions
toto <- list.files(paste0("Geographies/", clade, "/"))
toto <- toto[grep("_93271_Simpson_final.shp", toto)]
bioregions <- st_read(paste("Geographies/", clade, "/", toto, sep = ""))
# put bioregions in the right order
bioregions <- bioregions[order(bioregions$Bioregion),]
rm(toto)

model <- "DEC"


WD <- 12; HG <- 30 # width and height of pdfs


load(paste0("BioGeoBEARS/", clade, "/", subfolder, clade, 
            "_", model, ".Rdata"))
assign("res0", res)
load(paste0("BioGeoBEARS/", clade, "/", subfolder, clade, 
            "_", model, "+x.Rdata"))
assign("resx", res)

for(j in traits_incl){
  # load models
  load(paste0("BioGeoBEARS/", clade, "/", subfolder, clade, 
              "_", model, "+m2", j, ".Rdata"))
  assign("resm2", res)
  load(paste0("BioGeoBEARS/", clade, "/", subfolder, clade, 
              "_", model, "+m2+x", j, ".Rdata"))
  assign("resm2x", res)
  
  # for colour palette
  # resDEC$inputs$states_list <- resDEC$inputs$lists_of_states_lists_0based[[1]]
  # resDECx$inputs$states_list <- resDECx$inputs$lists_of_states_lists_0based[[1]]
  # resDECm2$inputs$states_list <- resDECm2$inputs$lists_of_states_lists_0based[[1]]
  # resDECm2x$inputs$states_list <- resDECm2x$inputs$lists_of_states_lists_0based[[1]]
  
  analysis_titletxt1 <- c(model, paste0(model, "+x"),
                          paste0(model, "+m2"), paste0(model, "+m2+x"))
  results_object <- list(res0, resx, resm2, resm2x)
  names(results_object) <- c(paste0("res", model), paste0("res", model, "x"),
                             paste0("res", model, "m2"), paste0("res", model, "m2x"))
  
  pdffn <- paste0(output_results, "/", clade, "/", clade, "_Biogeographic_estimations_", 
                  sub("/", "", subfolder), ".pdf")
  # append trait to pdf filename
  pdffn <- sub(".pdf", paste0(j, ".pdf"), pdffn)
  
  if(sv.output == 1){
    pdf(file = pdffn, width = WD, height = HG)
    # choosing here 10 colors cause the nullstate is included in the biogeobears plots
    # plot(bioregions$geometry, col = inferno(10)[seq(10,2,by=-1)])
    # legend("bottom", legend = bioregions$Bioregion, pch = 22, 
    #        pt.bg = inferno(dim(bioregions)[1]+1)[seq(dim(bioregions)[1]+1,2,by=-1)], 
    #        cex = 0.8, ncol = dim(bioregions)[1]+1, 
    #        xjust = 0.5)
    colors_matrix <- get_colors_for_numareas(length(names(tipranges@df)))
    cols <- vector(mode = "numeric", length = ncol(colors_matrix))
    for(i in 1:ncol(colors_matrix)){
      cols[i] <- rgb(red = colors_matrix[1,i], green = colors_matrix[2,i], 
                     blue = colors_matrix[3,i], maxColorValue = 255)
    }
    plot(bioregions$geometry, col = cols)
    legend("bottom", legend = bioregions$Bioregion, pch = 22, 
           pt.bg = cols, 
           cex = 0.8, ncol = dim(bioregions)[1]+1, 
           xjust = 0.5)
    rm(colors_matrix, cols)
    
    
    for(i in 1:length(results_object)){
      # if trait-dependent: plot ranges and ancestral trait states
      if(length(grep("m2", names(results_object)[i])) > 0){
        my_plot_BioGeoBEARS_results(results_object[[i]], trait_dep = TRUE, 
                                    output_plot = "ances_ranges", 
                                    analysis_titletxt = analysis_titletxt1[i], 
                                    plotwhat = "text", label.offset = 0.45, tipcex = 0.3, 
                                    statecex = 0.5, 
                                    titlecex = 0.8, plotlegend = FALSE, 
                                    addl_params = list("j", "x"),
                                    include_null_range = TRUE, tr = ladderize(tree), 
                                    tipranges = tipranges, 
                                    colors_list_for_states = NULL)
        my_plot_BioGeoBEARS_results(results_object[[i]], trait_dep = TRUE, 
                                    output_plot = "ances_ranges", 
                                    analysis_titletxt = analysis_titletxt1[i], 
                                    plotwhat = "pie", label.offset = 0.45, 
                                    tipcex = 0.3, statecex = 0.5, 
                                    titlecex = 0.8, plotlegend = FALSE, 
                                    addl_params = list("j", "x"),
                                    include_null_range = TRUE, tr = ladderize(tree),
                                    tipranges = tipranges, 
                                    colors_list_for_states = NULL)
        my_plot_BioGeoBEARS_results(results_object[[i]], trait_dep = TRUE, 
                                    output_plot = "ances_traits", 
                                    analysis_titletxt = analysis_titletxt1[i], 
                                    plotwhat = "text", label.offset = 0.45, tipcex = 0.3, 
                                    statecex = 0.5, 
                                    titlecex = 0.8, plotlegend = TRUE, 
                                    addl_params = list("j", "x"),
                                    include_null_range = TRUE, tr = ladderize(tree), 
                                    tipranges = tipranges, 
                                    colors_list_for_states = NULL)
        my_plot_BioGeoBEARS_results(results_object[[i]], trait_dep = TRUE, 
                                    output_plot = "ances_traits", 
                                    analysis_titletxt = analysis_titletxt1[i], 
                                    plotwhat = "pie", label.offset = 0.45, 
                                    tipcex = 0.3, statecex = 0.5, 
                                    titlecex = 0.8, plotlegend = FALSE, 
                                    addl_params = list("j", "x"),
                                    include_null_range = TRUE, tr = ladderize(tree), 
                                    tipranges = tipranges, 
                                    colors_list_for_states = NULL)
      } else{
        # plot legend only colours for single areas. Composite ranges are a 
        # combination of single area colours
        # colors_legend(c("NA", paste0(bioregions$Bioregion, ": ", bioregions$Names)),
        #               colors_list_for_states_man[1:(dim(bioregions)[1]+1)])
        my_plot_BioGeoBEARS_results(results_object[[i]], trait_dep = FALSE, 
                                    output_plot = "ances_ranges", 
                                    analysis_titletxt = analysis_titletxt1[i], 
                                    plotwhat = "text", label.offset = 0.45, 
                                    tipcex = 0.3, statecex = 0.5, 
                                    titlecex = 0.8, plotlegend = FALSE, 
                                    addl_params = list("j", "x"),
                                    include_null_range = TRUE, tr = ladderize(tree), 
                                    tipranges = tipranges, 
                                    colors_list_for_states = NULL)
        my_plot_BioGeoBEARS_results(results_object[[i]], trait_dep = FALSE, 
                                    output_plot = "ances_ranges", 
                                    analysis_titletxt = analysis_titletxt1[i], 
                                    plotwhat = "pie", label.offset = 0.45, 
                                    tipcex = 0.3, statecex = 0.5, 
                                    titlecex = 0.8, plotlegend = FALSE, 
                                    addl_params = list("j", "x"),
                                    include_null_range = TRUE, tr = ladderize(tree), 
                                    tipranges = tipranges, 
                                    colors_list_for_states = NULL)
        
        
      }
      
    }
    
    dev.off()  
  }
}


  
  


