## Setting up and running the BioGeoBEARS analyses
# Sarah-Sophie Weil
# 06/01/2020


## what you need:
# - a phylogeny (.nwk format) containing only species of interest (in Data/BioGeoBEARS)
# - a geography .data file with areas occupied by species (in Data/Geographies)
# - a timeperiods .txt file in which timeperiods for stratified analysis are given (in 
#   Data/Geographies)
# - a manual dispersal multiplier matrix (.txt, in Data/Geographies)
# - an area allowed .txt file in which existing areas at different time periods are specified
#   (in Data/Geographies)
# - geographic distances between regions if you want to do the +x variant (in Data/Geographies)
# - trait data if you want to run trait-dependent models (in Data/Traits)
# - the scripts 5_2_Function_run_Biogeobears in which the actual biogeoBEARS code is run



## what you get:
# - a BioGeoBEARS .Rdata output in Data/BioGeoBEARS



## Some important notes:
# - to compare trait-dependent and trait-independent models, the trait data needs to be 
# estimated separately. So the parameters t12 and t21 are estimated using a single 
# area geography (in Data/Geographies) and setting all parameters to zero or 1
# - the complex models are build hierarchically, 
# i.e. parameter estimation starts from parameter estimation of the previous simpler runs.
# This means that the simpler models need to be run first, otherwise the scripts won't work





# clear environment
rm(list = ls())



#### Preparation ----
# set up working directory and output directory
wd <- "../../Data/"
output_data <- "../Data"
output_results <- "../Outputs"
setwd(wd)
list.files()

# for installing BioGeoBEARS:
# install.packages("optimx", dependencies=TRUE, repos="http://cran.rstudio.com")
# install.packages("phylobase", dependencies=TRUE, repos="http://cran.rstudio.com")
# install.packages("devtools")
# library(devtools)
# devtools::install_github(repo="nmatzke/BioGeoBEARS", INSTALL_opts="--byte-compile",
#  dependencies=FALSE)

# load packages
library(optimx)      # 2020.4.2
library(FD)          # 1.0.12
library(snow)        # 0.4-3
library(BioGeoBEARS) # 1.1.2
library(phylobase)   # 0.8.10
library(phangorn)    # 2.6.2
library(strap)       # 1.4
library(phylolm)     # 2.6
library(cladoRcpp)   # 0.15.1    # for function numstates_from_numareas
library(parallel)    # 3.6.3     # for function detectCores
# if you need help installing the BioGeoBEARS package: http://phylo.wikidot.com/biogeobears#toc3



sv.output <- 1 # save if 1
runslow <- TRUE 
# the first analysis needs to be runslow <- TRUE, afterwards can be set to FALSE: then
# R objects created in previous analysis will be called... much faster because no reanalysis
clade <- "Chamaeleonidae" # this is going to be added before each output
mult <- "full0.1" 
# define which manual dispersal multiplier file you want, can be null, simple, medium,
# full0.1, full0.5
base_model <- "DEC" # can be DEC, BAYAREA, DIVA
param <- "m2x" # can be 0, x, m2, m2x
traits_incl <- "combo33_new" 
# can be 0 (for trait-independent), habitat, body_sizePFA, LH, LH_first_25, LH_first_75, 
# LH_first_and_last_25, combo33_new
root_constraint <- "Yes"  # Yes or No

core_parallel <- detectCores()-7
# number of cores to use for BioGeoBEARS model, use detectCores() to test

# define the script that contains the actual BioGeoBEARS model and which will be sourced
# at the end of this script
source_biogeo <- paste0("../Scripts/Cluster_Scripts/5_2_Function_run_Biogeobears_",
                        base_model, param, ".R")
print(source_biogeo)





#### Loading required files ----

# path to tree
trfn <- paste("BioGeoBEARS/", clade, "/", clade, "_tree_final.nwk", sep = "")
# load it
tree <- read.tree(file = trfn)

#plot(tree)
#axisPhylo(side = 1)
length(tree$tip.label)

# from phylowiki: Now, LOOK at the plot of the phylogeny. 
# If the x-axis spans, say, 5 or 10 or 50 time units 
# (typically with phylogenies, the units are millions of years), 
# then your tree is not super-short. 
# But, if the x-axis spans only, say 0.06 units, then your tree is super-short, then you 
# probably have a problem.
if(max(tree$edge.length) < 4.5){
  stop("your tree is very probably super short!")
}

# also: individual branches should be > 0.000001, so 
if(min(tree$edge.length) < 0.000001){
  stop("you have a super short tree!")
}

# also: tree needs to be binary (no polytomies). If that's the case: 
# resolve them and save a new tree
if(!is.binary(tree)){
  if(!file.exists(paste(output_data, "/BioGeoBEARS/", clade, "/", clade, 
                        "_tree_final_binary.nwk", sep = ""))){
    tree <- multi2di(tree)
    if(min(tree$edge.length) < 0.000001){
      # negative edge lengths happened when I did this, so I'm changing them manually here
      tree$edge.length[which(tree$edge.length < 0.00001)] <- 0.00001
    }
    write.tree(tree, paste(output_data, "/BioGeoBEARS/", clade, "/", 
                           clade, "_tree_final_binary.nwk", sep = ""))
  }
  trfn <- paste(output_data, "/BioGeoBEARS/", clade, "/", clade, "_tree_final_binary.nwk",
                sep = "")
}


# load geography file
geogfn <- paste(output_data, "/Geographies/", clade, "/", clade, "_geography.data", sep = "")
moref(geogfn)   # Look at the raw geography text file:

# Look at your geographic range data:
tipranges <- getranges_from_LagrangePHYLIP(lgdata_fn = geogfn)

# Set the maximum number of areas any species may occupy
# from phylo wiki:
# Computationally, the number of states/ranges should be roughly:
# - less than 1000 if you want the analysis to run in under a day
# - less than 1500 if you want the analysis to run in under a week, and
# - less than 2500 if you want it to run at all
# important: adding a binary trait doubles the state matrix

max_range_size <- 3
numstates_from_numareas(numareas = 9, maxareas = max_range_size, include_null_range = TRUE)

# changing manually the state space:
# keeping all the single-area and 2-area ranges, the observed 4-area range and any 
# other 4-area ranges 
# that seem particularly important/plausible, any 3-area ranges that are subsets 
# of the 4-area ranges.

areas = getareas_from_tipranges_object(tipranges)

# This is the list of states/ranges, where each state/range
# is a list of areas, counting from 0
# 
states_list_0based = rcpp_areas_list_to_states_list(areas=areas, maxareas=2, 
                                                    include_null_range=TRUE)

# How many states/ranges: 46
length(states_list_0based)

# Make the list of ranges
ranges_list = NULL
for (i in 1:length(states_list_0based)){    
  if ( (length(states_list_0based[[i]]) == 1) && (is.na(states_list_0based[[i]])) )
  {
    tmprange = "_"
  } else {
    tmprange = paste(areas[states_list_0based[[i]]+1], collapse="")
  }
  ranges_list = c(ranges_list, tmprange)
}

# Look at the ranges list
ranges_list

# How many states/ranges: 46
length(ranges_list)
# library(sf) # 0.9.8, GEOS 3.8.0, GDAL 3.0.4, PROJ 6.3.1
# bioregions <- st_read(paste0("Geographies/Chamaeleonidae/",
#                              "Chamaeleonidae_bioregions_upgma_const_93271_Simpson_final.shp"))
# library(mapview) # 2.9.0
# mapview(bioregions)
# Let's add some ranges
to_add <- c("257", "125", "127", "128", "189", "289", "246", "236", "369", 
            "349", "348", "247", "267", "124")

to_add_list <- list(c(2,5,7)-1, c(1,2,5)-1, c(1,2,7)-1, c(1,2,8)-1, c(1,8,9)-1, 
                    c(2,8,9)-1, c(2,4,6)-1, 
                    c(2,3,6)-1, c(3,6,9)-1, c(3,4,9)-1, c(3,4,8)-1, c(2,4,7)-1, 
                    c(2,6,7)-1, c(1,2,4)-1)
# the states list is 0 based -> therefore I need to do -1 everywhere
# rm(bioregions)
ranges_list_NEW = c(ranges_list, to_add)
length(ranges_list_NEW)     # now 61

states_list_0based_NEW = c(states_list_0based, to_add_list)
length(states_list_0based_NEW)     # now 61
rm(ranges_list, ranges_list_NEW, to_add, to_add_list, tmprange, areas, states_list_0based, i)


# create filenames of other files that are going to be needed for the analysis
if(mult %in% c("full0.1", "full0.5")){
  if(length(grep("0.1", mult)) > 0){
    mult1 <- 0.1
  } else if(length(grep("0.5", mult)) > 0){
    mult1 <- 0.5
  } else{
    stop("There's a problem here. Check your manual dispersal multiplier matrices and the path.")
  }
  dispersal_multipliers_fn <- paste("Geographies/", clade, "/", clade,
                                    "_manual_dispersal_multipliers", mult1, ".txt", sep = "")
} else if(mult == "simple"){
  dispersal_multipliers_fn <- paste("Geographies/", clade, "/", clade,
                                    "_manual_dispersal_multipliers0.5_simple.txt", sep = "")
} else if(mult == "medium"){
  dispersal_multipliers_fn <- paste("Geographies/", clade, "/", clade,
                                    "_manual_dispersal_multipliers0.5_medium.txt", sep = "")
}
# mult = null is implicitly taken into account. There is just difference in dispersal between
# regions, so no dispersal multiplier matrix is necessary  

timesfn <- paste("Geographies/", clade, "/", clade, "_timeperiods.txt", sep = "")
areas_allowed_fn <- paste("Geographies/", clade, "/", clade, "_areas_allowed.txt", sep = "")

# there's a problem with the geographic distance matrices: 
# BioGeoBEARS doesn't accept a 0 distance anywhere but on the diagonal
# so in the past, when there were regions that didn't exit and I put a 0 distance, 
# it gives an error.
# Putting all zeroes to 0.0001 (only if it hasn't been done before in a previous run)
if(!file.exists(paste("Geographies/", clade, "/", clade,
                      "_geographic_distances_final.txt", sep = ""))){
  geogdist_fn <- paste(output_data, "/Geographies/", clade, "/", clade, 
                       "_geographic_distances_new.txt", sep = "")
  geog_dist <- read.delim(geogdist_fn, header = FALSE, sep = "\t", 
                          blank.lines.skip = FALSE, stringsAsFactors = FALSE)
  # delete last row (containing END) because the END transforms the whole column into characters
  geog_dist <- geog_dist[1:(dim(geog_dist)[1]-1),]
  geog_dist[,1] <- as.numeric(geog_dist[,1])
  geog_dist[geog_dist == 0] <- 0.0001
  
  write.table(geog_dist[1:(dim(geog_dist)[2]+1),], 
              file = paste(output_data, "/Geographies/", clade, "/", clade,
                           "_geographic_distances_final.txt", sep = ""),
              row.names = FALSE, col.names = FALSE, sep = "\t")
  if(dim(geog_dist)[1] > (dim(geog_dist)[2]+1)){
    # figuring out how many matrices there are (since they are square, 
    # one matrix has as many rows as columns +2 (for header and empty row))
    for(j in 2:(dim(geog_dist)[1]/(dim(geog_dist)[2]+2))){
      cat("\n", file = paste(output_data, "/Geographies/", clade, "/", clade,
                             "_geographic_distances_final.txt", sep = ""), append = TRUE)
      # figuring out which rows need to be appended for which j
      write.table(geog_dist[((j-1)*(dim(geog_dist)[2]+2)+1):((j-1)*(dim(geog_dist)[2]+2)+
                                                               1+dim(geog_dist)[2]),], 
                  file = paste(output_data, "/Geographies/", clade, "/", 
                               clade, "_geographic_distances_final.txt", sep = ""),
                  row.names = FALSE, col.names = FALSE, append = TRUE, sep = "\t")
    }
  }
  cat("\nEND\n", file = paste(output_data, "/Geographies/", clade, "/", clade,
                              "_geographic_distances_final.txt", sep = ""), append = TRUE)
  rm(j)
  rm(geog_dist)
}

geogdist_fn <- paste("Geographies/", clade, "/", clade, 
                     "_geographic_distances_final.txt", sep = "")

if(traits_incl != 0){
  trait_fn <- paste("Traits/", clade, "/", clade, "_traits_", traits_incl, ".data", sep = "")
  trait_values <- getranges_from_LagrangePHYLIP(lgdata_fn = trait_fn)
  trait_values
}




#### Prepare "fossil" data ----
if(root_constraint == "Yes"){
  # I'm going to constrain the root node because before, the model estimated a 
  # wide-spread ancestor while 
  # the literature quite clearly indicates an African origin for chameleons
  
  tree_orig <- read.tree(trfn)

  # figure out which internal nodes I need to keep as "fossils" for the present analysis
  # to do this, I'm figuring out which nodelabel the root has
  toto <- as(tree_orig, "phylo4")
  toto
  root_node <- nodeId(toto, "root")
  
  # If traits are included, the size of the matrix is doubled; first half is 
  # ranges+trait state 1, second half is ranges + trait state2
  if(length(grep("m2", param)) == 0){
    
    areas = getareas_from_tipranges_object(tipranges)
    states_list_0based = rcpp_areas_list_to_states_list(areas=areas, maxareas=max_range_size, 
                                                        include_null_range=TRUE)
    relprobs_matrix_for_internal_states <- matrix(0, nrow = 1, 
                                                  ncol = length(states_list_0based))
    rm(states_list_0based, areas)
    
  } else{
    areas = getareas_from_tipranges_object(tipranges)
    states_list_0based = rcpp_areas_list_to_states_list(areas=areas, maxareas=max_range_size, 
                                                        include_null_range=TRUE)
    relprobs_matrix_for_internal_states <- matrix(0, nrow = 1, 
                                                  ncol = length(states_list_0based)*2)
    rm(states_list_0based, areas)
  }
  
  rm(tree_orig, toto)
  
  

  if(length(grep("m2", param)) == 0){
    # constraining root to Central Africa (which is region 2 but since there's 
    # the nullrange, it's element 3)
    relprobs_matrix_for_internal_states[3] <- 1
  } else{
    # If you're running a trait-dependent model, you can't constrain the ancestral range
    # without constraining the root node. Depending on the ancestral trait state,
    # the constraint is in the second or first half of the matrix.
    if(traits_incl %in% c("LH", "LH_first_25", "LH_first_and_last_25", "combo33_new")){
      relprobs_matrix_for_internal_states[3+length(relprobs_matrix_for_internal_states)/2] <- 1 
    } else{
      relprobs_matrix_for_internal_states[3] <- 1 
    }
    
  }
  
  print(relprobs_matrix_for_internal_states)
  if(sum(relprobs_matrix_for_internal_states) != 1){
    stop("Something went wrong with the root constraint.")
  }
  
  # looks good
  # so this is the matrix that I'm going to add to the BioGeoBEARS_run_object
}









#### Set up parameters and filenames ----
# doing this in a for-loop to create same filenames for DEC, DIVALIKE and BAYAREALIKE
# there are a few models here (+j) that I'm not going to run, but that doesn't matter
short <- c("DEC", "DIV", "BAY") 
long <- c("DEC", "DIVALIKE", "BAYAREALIKE")
# storing the filenames in lists cause otherwise it's cluttering the workspace
resfn_all <- analysis_titletxt <- vector(mode = "list", length = 3)
names(resfn_all) <- names(analysis_titletxt) <- long
if(mult == "null"){
  if(file.exists(paste0("BioGeoBEARS/", clade, "/MDMM_null"))){
    print("Subfolder exists already in BioGeoBEARS")
  } else {
    dir.create(file.path(paste0("BioGeoBEARS/", clade, "/MDMM_null")))
    print("Subfolder was successfully created")
  }
  
  for(i in 1:length(short)){
    resfn_all[[i]][1] <- paste0("BioGeoBEARS/", clade, "/MDMM_null/", clade, "_", long[i], 
                                "+traits_only.Rdata")
    resfn_all[[i]][2] <- paste0("BioGeoBEARS/", clade, "/MDMM_null/", clade, "_", long[i], 
                                ".Rdata")
    resfn_all[[i]][3] <- paste0("BioGeoBEARS/", clade, "/MDMM_null/", clade, "_", long[i], 
                                "+j.Rdata")
    resfn_all[[i]][4] <- paste0("BioGeoBEARS/", clade, "/MDMM_null/", clade, "_", long[i], 
                                "+x.Rdata")
    resfn_all[[i]][5] <- paste0("BioGeoBEARS/", clade, "/MDMM_null/", clade, "_", long[i], 
                                "+x+j.Rdata")
    resfn_all[[i]][6] <- paste0("BioGeoBEARS/", clade, "/MDMM_null/", clade, "_", long[i], 
                                "+m2.Rdata")
    resfn_all[[i]][7] <- paste0("BioGeoBEARS/", clade, "/MDMM_null/", clade, "_", long[i], 
                                "+m2+j.Rdata")
    resfn_all[[i]][8] <- paste0("BioGeoBEARS/", clade, "/MDMM_null/", clade, "_", long[i], 
                                "+m2+x.Rdata")
    resfn_all[[i]][9] <- paste0("BioGeoBEARS/", clade, "/MDMM_null/", clade, "_", long[i], 
                                "+m2+x+j.Rdata")
    names(resfn_all[[i]]) <- c(paste0("resfn_", short[i], "traits"), paste0("resfn_", 
                                                                            short[i]), 
                               paste0("resfn_", short[i], "j"),
                               paste0("resfn_", short[i], "x"), paste0("resfn_", short[i], "jx"), 
                               paste0("resfn_", short[i], "m2"),
                               paste0("resfn_", short[i], "m2j"), 
                               paste0("resfn_", short[i], "m2x"), 
                               paste0("resfn_", short[i], "m2jx"))
    
  }
} else if(mult == "full0.1"){
  if(file.exists(paste0("BioGeoBEARS/", clade, "/MDMM_full01"))){
    print("Subfolder exists already in BioGeoBEARS")
  } else {
    dir.create(file.path(paste0("BioGeoBEARS/", clade, "/MDMM_full01")))
    print("Subfolder was successfully created")
  }
  
  for(i in 1:length(short)){
    resfn_all[[i]][1] <- paste0("BioGeoBEARS/", clade, "/MDMM_full01/", clade, "_", long[i], 
                                "+traits_only.Rdata")
    resfn_all[[i]][2] <- paste0("BioGeoBEARS/", clade, "/MDMM_full01/", clade, "_", long[i], 
                                ".Rdata")
    resfn_all[[i]][3] <- paste0("BioGeoBEARS/", clade, "/MDMM_full01/", clade, "_", long[i], 
                                "+j.Rdata")
    resfn_all[[i]][4] <- paste0("BioGeoBEARS/", clade, "/MDMM_full01/", clade, "_", long[i], 
                                "+x.Rdata")
    resfn_all[[i]][5] <- paste0("BioGeoBEARS/", clade, "/MDMM_full01/", clade, "_", long[i], 
                                "+x+j.Rdata")
    resfn_all[[i]][6] <- paste0("BioGeoBEARS/", clade, "/MDMM_full01/", clade, "_", long[i], 
                                "+m2.Rdata")
    resfn_all[[i]][7] <- paste0("BioGeoBEARS/", clade, "/MDMM_full01/", clade, "_", long[i], 
                                "+m2+j.Rdata")
    resfn_all[[i]][8] <- paste0("BioGeoBEARS/", clade, "/MDMM_full01/", clade, "_", long[i], 
                                "+m2+x.Rdata")
    resfn_all[[i]][9] <- paste0("BioGeoBEARS/", clade, "/MDMM_full01/", clade, "_", long[i], 
                                "+m2+x+j.Rdata")
    names(resfn_all[[i]]) <- c(paste0("resfn_", short[i], "traits"), paste0("resfn_", 
                                                                            short[i]), 
                               paste0("resfn_", short[i], "j"),
                               paste0("resfn_", short[i], "x"), paste0("resfn_", short[i], "jx"), 
                               paste0("resfn_", short[i], "m2"),
                               paste0("resfn_", short[i], "m2j"), 
                               paste0("resfn_", short[i], "m2x"), 
                               paste0("resfn_", short[i], "m2jx"))
    
  }
} else if(mult == "full0.5"){
  if(file.exists(paste0("BioGeoBEARS/", clade, "/MDMM_full05"))){
    print("Subfolder exists already in BioGeoBEARS")
  } else {
    dir.create(file.path(paste0("BioGeoBEARS/", clade, "/MDMM_full05")))
    print("Subfolder was successfully created")
  }
  
  for(i in 1:length(short)){
    resfn_all[[i]][1] <- paste0("BioGeoBEARS/", clade, "/MDMM_full05/", clade, "_", long[i], 
                                "+traits_only.Rdata")
    resfn_all[[i]][2] <- paste0("BioGeoBEARS/", clade, "/MDMM_full05/", clade, "_", long[i], 
                                ".Rdata")
    resfn_all[[i]][3] <- paste0("BioGeoBEARS/", clade, "/MDMM_full05/", clade, "_", long[i], 
                                "+j.Rdata")
    resfn_all[[i]][4] <- paste0("BioGeoBEARS/", clade, "/MDMM_full05/", clade, "_", long[i], 
                                "+x.Rdata")
    resfn_all[[i]][5] <- paste0("BioGeoBEARS/", clade, "/MDMM_full05/", clade, "_", long[i], 
                                "+x+j.Rdata")
    resfn_all[[i]][6] <- paste0("BioGeoBEARS/", clade, "/MDMM_full05/", clade, "_", long[i], 
                                "+m2.Rdata")
    resfn_all[[i]][7] <- paste0("BioGeoBEARS/", clade, "/MDMM_full05/", clade, "_", long[i], 
                                "+m2+j.Rdata")
    resfn_all[[i]][8] <- paste0("BioGeoBEARS/", clade, "/MDMM_full05/", clade, "_", long[i], 
                                "+m2+x.Rdata")
    resfn_all[[i]][9] <- paste0("BioGeoBEARS/", clade, "/MDMM_full05/", clade, "_", long[i], 
                                "+m2+x+j.Rdata")
    names(resfn_all[[i]]) <- c(paste0("resfn_", short[i], "traits"), paste0("resfn_", 
                                                                            short[i]), 
                               paste0("resfn_", short[i], "j"),
                               paste0("resfn_", short[i], "x"), paste0("resfn_", short[i], "jx"), 
                               paste0("resfn_", short[i], "m2"),
                               paste0("resfn_", short[i], "m2j"), 
                               paste0("resfn_", short[i], "m2x"), 
                               paste0("resfn_", short[i], "m2jx"))
    
  }
} else if(mult == "simple"){
  if(file.exists(paste0("BioGeoBEARS/", clade, "/MDMM_simple"))){
    print("Subfolder exists already in BioGeoBEARS")
  } else {
    dir.create(file.path(paste0("BioGeoBEARS/", clade, "/MDMM_simple")))
    print("Subfolder was successfully created")
  }
  
  for(i in 1:length(short)){
    resfn_all[[i]][1] <- paste0("BioGeoBEARS/", clade, "/MDMM_simple/", clade, "_", long[i], 
                                "+traits_only.Rdata")
    resfn_all[[i]][2] <- paste0("BioGeoBEARS/", clade, "/MDMM_simple/", clade, "_", long[i], 
                                ".Rdata")
    resfn_all[[i]][3] <- paste0("BioGeoBEARS/", clade, "/MDMM_simple/", clade, "_", long[i], 
                                "+j.Rdata")
    resfn_all[[i]][4] <- paste0("BioGeoBEARS/", clade, "/MDMM_simple/", clade, "_", long[i], 
                                "+x.Rdata")
    resfn_all[[i]][5] <- paste0("BioGeoBEARS/", clade, "/MDMM_simple/", clade, "_", long[i], 
                                "+x+j.Rdata")
    resfn_all[[i]][6] <- paste0("BioGeoBEARS/", clade, "/MDMM_simple/", clade, "_", long[i], 
                                "+m2.Rdata")
    resfn_all[[i]][7] <- paste0("BioGeoBEARS/", clade, "/MDMM_simple/", clade, "_", long[i], 
                                "+m2+j.Rdata")
    resfn_all[[i]][8] <- paste0("BioGeoBEARS/", clade, "/MDMM_simple/", clade, "_", long[i], 
                                "+m2+x.Rdata")
    resfn_all[[i]][9] <- paste0("BioGeoBEARS/", clade, "/MDMM_simple/", clade, "_", long[i], 
                                "+m2+x+j.Rdata")
    names(resfn_all[[i]]) <- c(paste0("resfn_", short[i], "traits"), paste0("resfn_", 
                                                                            short[i]), 
                               paste0("resfn_", short[i], "j"),
                               paste0("resfn_", short[i], "x"), paste0("resfn_", short[i], "jx"), 
                               paste0("resfn_", short[i], "m2"),
                               paste0("resfn_", short[i], "m2j"), 
                               paste0("resfn_", short[i], "m2x"), 
                               paste0("resfn_", short[i], "m2jx"))
    
  }
} else if(mult == "medium"){
  if(file.exists(paste0("BioGeoBEARS/", clade, "/MDMM_medium"))){
    print("Subfolder exists already in BioGeoBEARS")
  } else {
    dir.create(file.path(paste0("BioGeoBEARS/", clade, "/MDMM_medium")))
    print("Subfolder was successfully created")
  }
  
  for(i in 1:length(short)){
    resfn_all[[i]][1] <- paste0("BioGeoBEARS/", clade, "/MDMM_medium/", clade, "_", long[i], 
                                "+traits_only.Rdata")
    resfn_all[[i]][2] <- paste0("BioGeoBEARS/", clade, "/MDMM_medium/", clade, "_", long[i], 
                                ".Rdata")
    resfn_all[[i]][3] <- paste0("BioGeoBEARS/", clade, "/MDMM_medium/", clade, "_", long[i], 
                                "+j.Rdata")
    resfn_all[[i]][4] <- paste0("BioGeoBEARS/", clade, "/MDMM_medium/", clade, "_", long[i], 
                                "+x.Rdata")
    resfn_all[[i]][5] <- paste0("BioGeoBEARS/", clade, "/MDMM_medium/", clade, "_", long[i], 
                                "+x+j.Rdata")
    resfn_all[[i]][6] <- paste0("BioGeoBEARS/", clade, "/MDMM_medium/", clade, "_", long[i], 
                                "+m2.Rdata")
    resfn_all[[i]][7] <- paste0("BioGeoBEARS/", clade, "/MDMM_medium/", clade, "_", long[i], 
                                "+m2+j.Rdata")
    resfn_all[[i]][8] <- paste0("BioGeoBEARS/", clade, "/MDMM_medium/", clade, "_", long[i], 
                                "+m2+x.Rdata")
    resfn_all[[i]][9] <- paste0("BioGeoBEARS/", clade, "/MDMM_medium/", clade, "_", long[i], 
                                "+m2+x+j.Rdata")
    names(resfn_all[[i]]) <- c(paste0("resfn_", short[i], "traits"), paste0("resfn_", 
                                                                            short[i]), 
                               paste0("resfn_", short[i], "j"),
                               paste0("resfn_", short[i], "x"), paste0("resfn_", short[i], "jx"), 
                               paste0("resfn_", short[i], "m2"),
                               paste0("resfn_", short[i], "m2j"), 
                               paste0("resfn_", short[i], "m2x"), 
                               paste0("resfn_", short[i], "m2jx"))
    
  }
}
rm(short, long, i)






#### Run BiogeoBEARS script ----
# !!!! Takes a while to run if runslow = TRUE (several hours/days) !!!!
t5 <- proc.time()
source(source_biogeo)
t6 <- proc.time()

rm(areas_allowed_fn, dispersal_multipliers_fn, geogdist_fn, geogfn, max_range_size, 
   timesfn, trfn)

if(traits_incl != 0){
  rm(trait_fn, trait_values)
}



